package me.km.scheduler;

import java.util.ConcurrentModificationException;
import java.util.HashMap;
import java.util.LinkedList;
import me.hammerle.snuviscript.code.ISnuviScheduler;
import me.km.module.Module;
import net.minecraft.util.text.TextFormatting;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

@SideOnly(Side.SERVER)
public class SnuviScheduler extends Module implements ISnuviScheduler
{
    private boolean checker;
    private int counter;
    private final LinkedList<SnuviTask> addQueue;
    private final LinkedList<Integer> removeQueue;
    private final HashMap<Integer, SnuviTask> tasks;
    private final AsyncWorker worker = new AsyncWorker();
    private boolean noAdding = false;
    
    public SnuviScheduler(String prefix, TextFormatting color)
    {
        super(prefix, color);
        counter = 0;
        tasks = new HashMap<>();
        addQueue = new LinkedList<>();
        removeQueue = new LinkedList<>();
        worker.start();
    }
    
    public void stop()
    {
        noAdding = true;
        long endTime = System.nanoTime() + 60_000_000_000l;
        System.out.println("Waiting until end of synchronized tasks ...");
        while(!tasks.isEmpty() && System.nanoTime() < endTime)
        {
            tick();
        }
        System.out.println("Waiting until end of asynchronous tasks ...");
        while(worker.hasWork() && System.nanoTime() < endTime)
        {
            try
            {
                Thread.sleep(250);
            }
            catch(InterruptedException ex)
            {
            }
        }
        System.out.println("the scheduler is now stopped");
    }
    
    public void cancelTask(int id)
    {
        removeQueue.add(id);
    }
    
    public int scheduleRepeatingTask(Runnable r, long ticks, long rtimer)
    {
        if(noAdding)
        {
            System.out.println("scheduling tasks is not allowed on server stop");
            return -1;
        }
        synchronized(addQueue)
        {
            counter++;
            addQueue.add(new SnuviTask(counter, r, ticks, rtimer));
        }
        return counter;
    }
    
    @Override
    public int scheduleTask(Runnable r, long delay)
    { 
        return scheduleRepeatingTask(r, delay, 0);
    }
    
    @Override
    public int scheduleTask(Runnable r)
    { 
        return scheduleTask(r, 0);
    }
    
    public void tick()
    {
        worker.doWork();
        try
        {
            if(!removeQueue.isEmpty())
            {
                removeQueue.forEach(i -> tasks.remove(i));
                removeQueue.clear();
            }
            synchronized(addQueue)
            {
                if(!addQueue.isEmpty())
                {
                    addQueue.forEach(task -> tasks.put(task.getId(), task));
                    addQueue.clear();
                }
            }
            checker = false;
            long l = System.currentTimeMillis();
            tasks.values().removeIf(task -> 
            {
                if(checker)
                {
                    return false;
                }
                if(System.currentTimeMillis() - l > 25)
                {
                    checker = true;
                    this.sendWarningToConsole("Der Scheduler ist länger als 25 ms gelaufen.");
                    return false;
                }
                try
                {
                    return task.tick(noAdding);
                }
                catch(Exception ex)
                {
                    this.sendWarningToConsole("Scheduler-Exception - Task wird entfernt.");
                    ex.printStackTrace();
                    return true;
                }
            });
        }
        catch(ConcurrentModificationException ex)
        {
            this.sendWarningToConsole(ex.toString());
        }
    }
    
    public AsyncWorker getWorker()
    {
        return worker;
    }
}
