package me.km.snuviscript;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import me.km.KajetansMod;
import me.km.module.Module;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.UUID;
import me.hammerle.snuviscript.code.Script;
import me.hammerle.snuviscript.code.SnuviParser;
import me.km.chatchannel.ChatChannel;
import me.km.events.CustomEventCaller;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.text.TextFormatting;

public class ScriptModule extends Module
{
    private final SnuviParser parser;
    protected final SnuviLogger logger;
    private final HashMap<UUID, Integer> playerScript;   
    private final HashMap<Integer, List<UUID>> registeredPlayers; 
    private final HashSet<String> scriptCommands;
    private final ScriptBank bank = new ScriptBank();
    
    public ScriptModule(String prefix, TextFormatting color) 
    {
        super(prefix, color);
        
        logger = new SnuviLogger();
        parser = new SnuviParser(logger, KajetansMod.scheduler);
        
        MinecraftFunctions.registerFunctions(parser);
        
        playerScript = new HashMap<>();
        registeredPlayers = new HashMap<>();
        scriptCommands = new HashSet<>();
        
        registerEvent(new ScriptEvents());
    }
    
    public SnuviParser getSnuviParser()
    {
        return parser;
    }
    
    public ScriptBank getDatabank()
    {
        return bank;
    }
    
    // -------------------------------------------------------------------------
    // custom commands
    // -------------------------------------------------------------------------
    
    public boolean isRegisteredScriptCommand(String s)
    {
        return scriptCommands.contains(s);
    }
    
    public void registerScriptCommand(String s)
    {
        if(s.equals("script"))
        {
            return;
        }
        scriptCommands.add(s);
    }
    
    public void unregisterScriptCommand(String s)
    {
        scriptCommands.remove(s);
    }
    
    public void clearScriptCommands()
    {
        scriptCommands.clear();
    }
    
    // -------------------------------------------------------------------------
    // player script
    // -------------------------------------------------------------------------

    public List<UUID> getPlayerList(int id)
    {
        return registeredPlayers.getOrDefault(id, Collections.EMPTY_LIST);
    }
    
    private List<UUID> getOrCreatePlayerList(int id)
    {
        List<UUID> list = registeredPlayers.get(id);
        if(list == null)
        {
            list = new ArrayList<>();
            registeredPlayers.put(id, list);
        }
        return list;
    }
    
    public boolean registerPlayer(Script sc, EntityPlayer p)
    {             
        if(playerScript.putIfAbsent(p.getUniqueID(), sc.getId()) != null)
        {
            return false;
        }       
        getOrCreatePlayerList(sc.getId()).add(p.getUniqueID());
        ScriptVars.setPlayerVars(sc, p); 
        parser.callEvent("player_join", sc, null, null);
        return true;
    } 
        
    public boolean isRegistered(EntityPlayer p)
    {
        return playerScript.containsKey(p.getUniqueID());
    }
    
    public Script getScript(EntityPlayer p)
    {
        // check if the player is mapped to a script
        Integer id = playerScript.get(p.getUniqueID());
        if(id == null)
        {
            return null;
        }
        // player mapping does not necessary mean the script is valid
        Script sc = parser.getScript(id);
        if(sc == null)
        {
            playerScript.remove(p.getUniqueID());
            ChatChannel.getDevChannel().sendWarning("Das ungültige Script von '" + p.getName() + "' wurde entfernt. (1)");
            return null;
        }
        return sc;
    }
    
    public boolean unregisterPlayer(Script callingScript, EntityPlayer p)
    {
        Integer id = playerScript.remove(p.getUniqueID());
        if(id == null)
        {
            return false;
        }
        List<UUID> list = registeredPlayers.get(id);
        if(list == null)
        {
            return false;
        }
        boolean b = list.remove(p.getUniqueID());
        if(list.isEmpty())
        {
            Script sc = parser.getScript(id);
            if(sc == null)
            {
                ChatChannel.getDevChannel().sendWarning("Das ungültige Script von '" + p.getName() + "' wurde entfernt. (2)");
                return true;
            }
            if(sc != callingScript)
            {
                parser.callEvent("player_kicked", sc, (innerSc) -> ScriptVars.setPlayerVars(innerSc, p), null);
            }
            parser.termSafe(sc);
        }
        return b;
    }
    
    public void clearPlayerRegistry()
    {
        playerScript.clear();
        registeredPlayers.clear();
    }

    // -------------------------------------------------------------------------
    // script starting
    // -------------------------------------------------------------------------
    
    public boolean startPlayerScript(EntityPlayer p, String... names)
    {
        if(isRegistered(p))
        {
            return false;
        }
        Arrays.setAll(names, i -> "scripts/" + names[i]);
        parser.startScript(false, sc -> 
        {
            int id = sc.getId();
            playerScript.put(p.getUniqueID(), id);
            getOrCreatePlayerList(id).add(p.getUniqueID());
        }, sc -> 
        {
            getPlayerList(sc.getId()).forEach(uuid -> playerScript.remove(uuid));
            registeredPlayers.remove(sc.getId());
            
            CustomEventCaller.removeScriptData(sc);
        }, ".txt", names);
        return true;
    }
    
    public void startScript(String... names)
    {
        Arrays.setAll(names, i -> "scripts/" + names[i]);
        parser.startScript(true, sc -> {}, sc -> CustomEventCaller.removeScriptData(sc), ".txt", names);
    }
}
