package me.km.utils;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import me.km.KajetansMod;
import me.km.world.ModTeleporter;
import me.km.entities.EntityItemProjectile;
import me.km.exception.PlayerNotFoundException;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.block.Block;
import net.minecraft.block.BlockButton;
import net.minecraft.block.BlockContainer;
import net.minecraft.block.BlockDoor;
import net.minecraft.block.BlockPistonMoving;
import net.minecraft.block.BlockTrapDoor;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.state.IBlockState;
import net.minecraft.command.ICommandSender;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.entity.projectile.EntityArrow;
import net.minecraft.entity.projectile.EntityFireball;
import net.minecraft.entity.projectile.EntityThrowable;
import net.minecraft.init.Blocks;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3d;
import net.minecraft.world.World;
import net.minecraft.network.play.server.SPacketEntityVelocity;
import net.minecraft.util.DamageSource;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.WorldServer;

public class Utils 
{    
    public static String colorMessage(String text, ICommandSender cs) 
    {
        if(KajetansMod.perms.hasPermission(cs, "color"))
        {
            text = text.replace('&', '§');
            return text;
        }
        text = text.replaceAll("&.", "");
        return text;
    }
    
    public static void setVelocity(Entity ent, double x, double y, double z)
    {
        ent.motionX = x;
        ent.motionY = y;
        ent.motionZ = z;
        updateVelocity(ent);
    }
    
    public static void addVelocity(Entity ent, double x, double y, double z)
    {
        ent.motionX += x;
        ent.motionY += y;
        ent.motionZ += z;
        updateVelocity(ent);
    }
    
    private static void updateVelocity(Entity ent)
    {
        if(ent instanceof EntityPlayerMP)
        {
            EntityPlayerMP p = (EntityPlayerMP) ent;
            SPacketEntityVelocity packet = new SPacketEntityVelocity(p);
            p.connection.sendPacket(packet);
        }
    }
    
    public static void scaleVelocity(Entity ent, double scale)
    {
        ent.motionX *= scale;
        ent.motionY *= scale;
        ent.motionZ *= scale;
        updateVelocity(ent);
    }
    
    public static <T extends Comparable<T>> T getStateValue(IBlockState state, IProperty<T> property)
    {
        try
        {
            return state.getValue(property);
        }
        catch(IllegalArgumentException ex)
        {
            return null;
        }
    }
             
    public static String formatString(String original)
    {
        if(original.length() == 0)
        {
            return original;
        }
        else if(original.length() == 1)
        {
            return original.toUpperCase();
        }
        String[] parts = original.split("_");
        for(int i = 0; i < parts.length; i++)
        {
            parts[i] = parts[i].substring(0, 1).toUpperCase() + parts[i].substring(1).toLowerCase();
        }
        return connectSpaces(parts, 0);
    }
   
    public static String connect(String[] array, int start, String c)
    {
        if(start >= array.length)
        {
            return "";
        }
        return Arrays.stream(array, start, array.length).collect(Collectors.joining(c));
    }
    
    public static String connectSpaces(String[] array, int start)
    {
        return connect(array, start, " ");
    } 
    
    public static EntityPlayer getPlayerFromProjectile(Entity ent)
    {
        if(ent instanceof EntityArrow)
        {
            EntityArrow arrow = (EntityArrow) ent;
            if(arrow.shootingEntity instanceof EntityPlayer)
            {
                return (EntityPlayer) arrow.shootingEntity;
            }
            return null;
        }
        else if(ent instanceof EntityThrowable)
        {
            EntityThrowable thrown = (EntityThrowable) ent;
            if(thrown.getThrower() instanceof EntityPlayer)
            {
                return (EntityPlayer) thrown.getThrower();
            }
            return null;
        }
        else if(ent instanceof EntityFireball)
        {
            EntityFireball fire = (EntityFireball) ent;
            if(fire.shootingEntity instanceof EntityPlayer)
            {
                return (EntityPlayer) fire.shootingEntity;
            }
            return null;
        }
        else if(ent instanceof EntityItemProjectile)
        {
            EntityItemProjectile item = (EntityItemProjectile) ent;
            if(item.getItemThrower() instanceof EntityPlayer)
            {
                return (EntityPlayer) item.getItemThrower();
            }
            return null;
        }
        return null;
    }
    
    public static EntityPlayer getDamager(DamageSource ds)
    {
        Entity ent = ds.getTrueSource();
        if(ent == null)
        {
            return null;
        }
        if(ent instanceof EntityPlayer)
        {
            return (EntityPlayer) ent;
        }
        return null;//getPlayerFromProjectile(ent);
    }

    public static void teleportEntity(Entity ent, Location l)
    {
        int dim = l.getWorld().provider.getDimension();
        if(ent instanceof EntityPlayerMP)
        {
            EntityPlayerMP p = (EntityPlayerMP) ent;
            if(ent.dimension != dim)
            {
                KajetansMod.server.getPlayerList().transferPlayerToDimension(p, dim, new ModTeleporter(KajetansMod.server.getWorld(dim)));
            }
            if(l.getYaw() != 0 || l.getPitch() != 0)
            {
                p.connection.setPlayerLocation(l.getX(), l.getY(), l.getZ(), l.getYaw(), l.getPitch());
            }
            else
            {
                p.connection.setPlayerLocation(l.getX(), l.getY(), l.getZ(), ent.rotationYaw, ent.rotationPitch);
            }
        }
        else
        {
            if(ent.dimension != dim)
            {
                WorldServer n = KajetansMod.server.getWorld(dim);
                KajetansMod.server.getPlayerList().transferEntityToWorld(ent, ent.dimension, KajetansMod.server.getWorld(ent.dimension), n, new ModTeleporter(n));
            }
            if(l.getYaw() != 0 && l.getPitch() != 0)
            {
                ent.setLocationAndAngles(l.getX(), l.getY(), l.getZ(), l.getYaw(), l.getPitch());
            }
            else
            {
                ent.setLocationAndAngles(l.getX(), l.getY(), l.getZ(), ent.rotationYaw, ent.rotationPitch);
            }
        }
    }
    
    private static AxisAlignedBB buildBoxAround(double x, double y, double z, double radius)
    {
        return new AxisAlignedBB(x - radius, y - radius, z - radius, x + radius, y + radius, z + radius);
    }
    
    @SuppressWarnings("unchecked")
    public static <T extends Entity> List<T> getEntities(World w, double x, double y, double z, double radius, Class<T> type)
    {       
        double sqareRadius = radius * radius;
        return w.getEntitiesWithinAABB(type, buildBoxAround(x, y, z, radius), ent -> ent.getDistanceSq(x, y, z) <= sqareRadius);
    }
    
    public static List<EntityPlayer> getPlayers(World w, double x, double y, double z, double radius)
    {       
        double sqareRadius = radius * radius;
        return w.getEntitiesWithinAABB(EntityPlayer.class, buildBoxAround(x, y, z, radius), p -> p.getDistanceSq(x, y, z) <= sqareRadius);
    }
    
    public static List<EntityPlayer> getPlayers(Entity not, double radius)
    {       
        double sqareRadius = radius * radius;
        return not.world.getEntitiesWithinAABB(EntityPlayer.class, buildBoxAround(not.posX, not.posY, not.posZ, radius), 
                p -> p != not && p.getDistanceSq(not) <= sqareRadius);
    }
    
    public static List<EntityLiving> getLiving(Entity not, double radius)
    {       
        double sqareRadius = radius * radius;
        return not.world.getEntitiesWithinAABB(EntityLiving.class, buildBoxAround(not.posX, not.posY, not.posZ, radius), 
                p -> p != not && p.getDistanceSq(not) <= sqareRadius);
    }

    public static <T extends Entity> T getEntity(World w, double x, double y, double z, double radius, Class<T> type)
    {
        return getEntities(w, x, y, z, radius, type).stream()
                .min((e1, e2) -> Double.compare(e1.getDistanceSq(x, y, z), e2.getDistanceSq(x, y, z)))
                .orElse(null);
    }
    
    private static boolean doesIntersect(AxisAlignedBB bound, Vec3d start, Vec3d unit)
    {
        double lowerX = Double.NEGATIVE_INFINITY;
        double upperX = Double.POSITIVE_INFINITY;
        if(unit.x != 0)
        {
            if(unit.x > 0)
            {
                lowerX = (bound.minX - start.x) / unit.x;
                upperX = (bound.maxX - start.x) / unit.x;
            }
            else
            {
                lowerX = (bound.maxX - start.x) / unit.x;
                upperX = (bound.minX - start.x) / unit.x;
            }
        }
        
        double lowerY = Double.NEGATIVE_INFINITY;
        double upperY = Double.POSITIVE_INFINITY;
        if(unit.y != 0)
        {
            if(unit.y > 0)
            {
                lowerY = (bound.minY - start.y) / unit.y;
                upperY = (bound.maxY - start.y) / unit.y;
            }
            else
            {
                lowerY = (bound.maxY - start.y) / unit.y;
                upperY = (bound.minY - start.y) / unit.y;
            }
        }
        
        double lowerZ = Double.NEGATIVE_INFINITY;
        double upperZ = Double.POSITIVE_INFINITY;
        if(unit.z != 0)
        {
            if(unit.z > 0)
            {
                lowerZ = (bound.minZ - start.z) / unit.z;
                upperZ = (bound.maxZ - start.z) / unit.z;
            }
            else
            {
                lowerZ = (bound.maxZ - start.z) / unit.z;
                upperZ = (bound.minZ - start.z) / unit.z;
            }
        }

        return Math.max(Math.max(lowerX, lowerY), lowerZ) < Math.min(Math.min(upperX, upperY), upperZ);
    }
    
    public static Vec3d getTargetVector(Entity ent, double range)
    {
        if(range > 128)
        {
            range = 128;
        }
        World w = ent.getEntityWorld();
        Vec3d start = ent.getPositionEyes(1.0f);
        Vec3d look = ent.getLookVec();
        Vec3d end = start.addVector(look.x * range, look.y * range, look.z * range);   
        RayTraceResult ray = w.rayTraceBlocks(start, end, false, true, true);
        if(ray == null || ray.hitVec == null)
        {
            return end;
        }
        return ray.hitVec;   
    }
    
    @SuppressWarnings("unchecked")
    public static <T extends Entity> T getTargetedEntity(EntityPlayer p, double radius, Class<T> type)
    {
        World w = p.getEntityWorld();
        Vec3d l = getTargetVector(p, radius);
        Vec3d eye = p.getPositionEyes(1.0f);
        Vec3d unit = new Vec3d(l.x - eye.x, l.y - eye.y, l.z - eye.z);
        
        List<Entity> col = w.getEntitiesWithinAABBExcludingEntity(p, new AxisAlignedBB(eye.x, eye.y, eye.z, l.x, l.y, l.z));
        col.removeIf(ent -> !type.isAssignableFrom(ent.getClass()));
        // removes entities, which do not intersect with the sight vector
        col.removeIf(ent -> !doesIntersect(ent.getEntityBoundingBox().grow(0.1), eye, unit));

        return (T) col.stream().sorted((Entity e1, Entity e2) -> 
        {
            return Double.compare(e1.getDistanceSq(eye.x, eye.y, eye.z), e2.getDistanceSq(eye.x, eye.y, eye.z));
        }).findFirst().orElse(null);
    }
    
    // -------------------------------------------------------------------------
    // Player-Tools
    // -------------------------------------------------------------------------

    public static BlockPos getTargetBlock(Entity ent, double range, boolean boundingBox)
    {
        if(range > 128)
        {
            range = 128;
        }
        World w = ent.getEntityWorld();
        Vec3d start = ent.getPositionEyes(1.0f);
        Vec3d look = ent.getLookVec();
        Vec3d end = start.addVector(look.x * range, look.y * range, look.z * range);   
        RayTraceResult ray = w.rayTraceBlocks(start, end, false, !boundingBox, true);
        if(ray == null || ray.getBlockPos() == null)
        {
            return new BlockPos(end);
        }
        return ray.getBlockPos();   
    }
    
    public static EntityPlayerMP getPlayerByName(String name) throws PlayerNotFoundException
    {
        String nameLower = name.toLowerCase();
        return KajetansMod.server.getPlayerList().getPlayers().stream()
                    .filter(pl -> pl.getName().toLowerCase().contains(nameLower))
                    .findFirst().orElseThrow(() -> new PlayerNotFoundException(name));
    }
}

